@@ -1,20 +1,11 @@
 // Copyright (c) 2001-2007  INRIA Sophia-Antipolis (France).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 3 of the License,
-// or (at your option) any later version.
+// This file is part of CGAL (www.cgal.org)
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
-//
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// $URL$
-// $Id$
-// SPDX-License-Identifier: LGPL-3.0+
+// $URL: https://github.com/CGAL/cgal/blob/v5.3.1/Number_types/include/CGAL/MP_Float.h $
+// $Id: MP_Float.h 12c7bb2 2020-05-19T11:24:12+02:00 Marc Glisse
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Sylvain Pion
@@ -36,6 +27,7 @@
 #include <iostream>
 #include <vector>
 #include <algorithm>
+#include <boost/operators.hpp>
 
 // MP_Float : multiprecision scaled integers.
 
@@ -86,7 +78,7 @@ std::pair<double,double> to_interval(const Quotient<MP_Float>&);
 MP_Float div(const MP_Float& n1, const MP_Float& n2);
 
 MP_Float gcd(const MP_Float& a, const MP_Float& b);
-  
+
 } //namespace INTERN_MP_FLOAT
 
 std::pair<double, int>
@@ -115,7 +107,13 @@ MP_Float operator*(const MP_Float &a, const MP_Float &b);
 MP_Float operator%(const MP_Float &a, const MP_Float &b);
 
 
-class MP_Float
+class MP_Float : boost::totally_ordered1<MP_Float
+#ifdef _MSC_VER
+                 , boost::ordered_ring_operators2<MP_Float, int
+                 , boost::ordered_ring_operators2<MP_Float, double
+                 > >
+#endif
+                 >
 {
 public:
   typedef short          limb;
@@ -163,12 +161,12 @@ public:
   {
     const unsigned int sizeof_limb=8*sizeof(limb);
     const limb2 mask = 0x0000ffff;
-   
-    //Note: For Integer type, if the destination type is signed, the value is unchanged 
+
+    //Note: For Integer type, if the destination type is signed, the value is unchanged
     //if it can be represented in the destination type)
     low = static_cast<limb>(l & mask); //extract low bits from l
     high= static_cast<limb>((l - low) >> sizeof_limb); //extract high bits from l
-    
+
     CGAL_postcondition ( l == low + ( static_cast<limb2>(high) << sizeof_limb ) );
   }
 
@@ -212,7 +210,7 @@ public:
     canonicalize();
   }
 
-  // 2019-11-30
+  // 2019-11-30, Jisu KIM
   // Temporarily added to resolve type cast ambiguity of type long int
   MP_Float(long i)
     : v(2), exp(0)
@@ -240,6 +238,22 @@ public:
   MP_Float& operator*=(const MP_Float &a) { return *this = *this * a; }
   MP_Float& operator%=(const MP_Float &a) { return *this = *this % a; }
 
+  friend bool operator<(const MP_Float &a, const MP_Float &b)
+  { return INTERN_MP_FLOAT::compare(a, b) == SMALLER; }
+
+  friend bool operator==(const MP_Float &a, const MP_Float &b)
+  { return (a.v == b.v) && (a.v.empty() || (a.exp == b.exp)); }
+
+#ifdef _MSC_VER
+  // Needed because without /permissive-, it makes hidden friends visible (from Quotient)
+  friend bool operator==(const MP_Float &a, int    b) { return a == MP_Float(b); }
+  friend bool operator==(const MP_Float &a, double b) { return a == MP_Float(b); }
+  friend bool operator< (const MP_Float &a, int    b) { return a <  MP_Float(b); }
+  friend bool operator< (const MP_Float &a, double b) { return a <  MP_Float(b); }
+  friend bool operator> (const MP_Float &a, int    b) { return a >  MP_Float(b); }
+  friend bool operator> (const MP_Float &a, double b) { return a >  MP_Float(b); }
+#endif
+
   exponent_type max_exp() const
   {
     return exponent_type(v.size()) + exp;
@@ -326,7 +340,7 @@ public:
   }
 
   // Accessory function that finds the least significant bit set (its position).
-  static unsigned short 
+  static unsigned short
   lsb(limb l)
   {
     unsigned short nb = 0;
@@ -382,30 +396,6 @@ inline
 void swap(MP_Float &m, MP_Float &n)
 { m.swap(n); }
 
-inline
-bool operator<(const MP_Float &a, const MP_Float &b)
-{ return INTERN_MP_FLOAT::compare(a, b) == SMALLER; }
-
-inline
-bool operator>(const MP_Float &a, const MP_Float &b)
-{ return b < a; }
-
-inline
-bool operator>=(const MP_Float &a, const MP_Float &b)
-{ return ! (a < b); }
-
-inline
-bool operator<=(const MP_Float &a, const MP_Float &b)
-{ return ! (a > b); }
-
-inline
-bool operator==(const MP_Float &a, const MP_Float &b)
-{ return (a.v == b.v) && (a.v.empty() || (a.exp == b.exp)); }
-
-inline
-bool operator!=(const MP_Float &a, const MP_Float &b)
-{ return ! (a == b); }
-
 MP_Float
 approximate_sqrt(const MP_Float &d);
 
@@ -418,7 +408,7 @@ approximate_division(const MP_Float &n, const MP_Float &d);
 template <> class Algebraic_structure_traits< MP_Float >
   : public Algebraic_structure_traits_base< MP_Float,
                                             Unique_factorization_domain_tag
-	    // with some work on mod/div it could be Euclidean_ring_tag
+            // with some work on mod/div it could be Euclidean_ring_tag
                                           >  {
   public:
 
@@ -479,19 +469,19 @@ template <> class Algebraic_structure_traits< MP_Float >
 
   typedef INTERN_AST::Mod_per_operator< Type > Mod;
 // Default implementation of Divides functor for unique factorization domains
-  // x divides y if gcd(y,x) equals x up to inverses 
-  class Divides 
+  // x divides y if gcd(y,x) equals x up to inverses
+  class Divides
     : public CGAL::cpp98::binary_function<Type,Type,bool>{
   public:
-    bool operator()( const Type& x,  const Type& y) const {  
+    bool operator()( const Type& x,  const Type& y) const {
       return internal::division(y,x).second == 0 ;
     }
-    // second operator computing q = x/y 
-    bool operator()( const Type& x,  const Type& y, Type& q) const {    
+    // second operator computing q = x/y
+    bool operator()( const Type& x,  const Type& y, Type& q) const {
       std::pair<Type,Type> qr = internal::division(y,x);
       q=qr.first;
       return qr.second == 0;
-      
+
     }
     CGAL_IMPLICIT_INTEROPERABLE_BINARY_OPERATOR_WITH_RT( Type , bool)
   };
@@ -542,7 +532,7 @@ template <> class Real_embeddable_traits< MP_Float >
 namespace INTERN_MP_FLOAT{
 
 //Sqrt_extension internally uses Algebraic_structure_traits
-template <class ACDE_TAG_, class FP_TAG>  
+template <class ACDE_TAG_, class FP_TAG>
 double
 to_double(const Sqrt_extension<MP_Float,MP_Float,ACDE_TAG_,FP_TAG> &x)
 {
@@ -577,7 +567,7 @@ to_double(const Sqrt_extension<MP_Float,MP_Float,ACDE_TAG_,FP_TAG> &x)
   std::pair<double, int> v3 = to_double_exp(d3);
   double scale3 = std::ldexp(1.0, n3.second - v3.second);
 
-  return ((n1.first / v1.first) * scale1) + 
+  return ((n1.first / v1.first) * scale1) +
          ((n2.first / v2.first) * scale2) *
          std::sqrt((n3.first / v3.first) * scale3);
 }
@@ -822,11 +812,11 @@ simplify_quotient(MP_Float & numerator, MP_Float & denominator)
 inline void simplify_root_of_2(MP_Float &/*a*/, MP_Float &/*b*/, MP_Float&/*c*/) {
 #if 0
   if(is_zero(a)) {
-  	simplify_quotient(b,c); return;
+          simplify_quotient(b,c); return;
   } else if(is_zero(b)) {
-  	simplify_quotient(a,c); return;
+          simplify_quotient(a,c); return;
   } else if(is_zero(c)) {
-  	simplify_quotient(a,b); return;
+          simplify_quotient(a,b); return;
   }
   MP_Float::exponent_type va = a.exp +
     (MP_Float::exponent_type) a.v.size();
@@ -877,16 +867,18 @@ public:
 };
 
 inline MP_Float min BOOST_PREVENT_MACRO_SUBSTITUTION(const MP_Float& x,const MP_Float& y){
-  return (x<=y)?x:y; 
+  return (x<=y)?x:y;
 }
 inline MP_Float max BOOST_PREVENT_MACRO_SUBSTITUTION(const MP_Float& x,const MP_Float& y){
-  return (x>=y)?x:y; 
+  return (x>=y)?x:y;
 }
 
 
 // Coercion_traits
 CGAL_DEFINE_COERCION_TRAITS_FOR_SELF(MP_Float)
 CGAL_DEFINE_COERCION_TRAITS_FROM_TO(int, MP_Float)
+CGAL_DEFINE_COERCION_TRAITS_FROM_TO(float, MP_Float)
+CGAL_DEFINE_COERCION_TRAITS_FROM_TO(double, MP_Float)
 
 
 } //namespace CGAL
